from rest_framework.test import APITransactionTestCase

from program import tests
from program.models import Schedule, Show


class NoteViewTestCase(tests.BaseMixin, APITransactionTestCase):
    reset_sequences = True

    show_beatbetrieb: Show
    schedule_beatbetrieb: Schedule
    show_musikrotation: Show
    schedule_musikrotation: Schedule

    def setUp(self) -> None:
        super().setUp()
        self.show_beatbetrieb = self._create_show("Beatbetrieb")
        self.schedule_beatbetrieb = self._create_schedule(self.show_beatbetrieb)
        self.show_musikrotation = self._create_show("Musikrotation", owners=[self.user_common])
        self.schedule_musikrotation = self._create_schedule(
            self.show_musikrotation, start_time="10:00", end_time="12:00"
        )

    def test_everyone_can_read_notes(self):
        self._create_note(self._create_timeslot(schedule=self.schedule_beatbetrieb))
        self._create_note(self._create_timeslot(schedule=self.schedule_musikrotation))
        res = self._get_client().get(self._url("notes"))
        self.assertEqual(len(res.data), 2)

    def test_common_users_can_create_notes_for_owned_shows(self):
        ts = self._create_timeslot(schedule=self.schedule_musikrotation)
        client = self._get_client(self.user_common)
        endpoint = self._url("notes")
        res = client.post(
            endpoint, self._create_random_note_content(timeslot_id=ts.id), format="json"
        )
        self.assertEqual(res.status_code, 201)

    def test_common_users_cannot_create_notes_for_foreign_shows(self):
        ts = self._create_timeslot(schedule=self.schedule_beatbetrieb)
        client = self._get_client(self.user_common)
        endpoint = self._url("notes")
        res = client.post(
            endpoint, self._create_random_note_content(timeslot_id=ts.id), format="json"
        )
        self.assertEqual(res.status_code, 404)

    def test_common_user_can_update_owned_shows(self):
        ts = self._create_timeslot(schedule=self.schedule_musikrotation)
        note = self._create_note(ts)
        client = self._get_client(self.user_common)
        new_note_content = self._create_random_note_content(title="meh")
        res = client.put(self._url("notes", note.id), new_note_content, format="json")
        self.assertEqual(res.status_code, 200)

    def test_common_user_cannot_update_notes_of_foreign_shows(self):
        ts = self._create_timeslot(schedule=self.schedule_beatbetrieb)
        note = self._create_note(ts)
        client = self._get_client(self.user_common)
        new_note_content = self._create_random_note_content(title="meh")
        res = client.put(self._url("notes", note.id), new_note_content, format="json")
        self.assertEqual(res.status_code, 404)

    def test_admin_can_create_notes_for_all_timeslots(self):
        timeslot = self._create_timeslot(schedule=self.schedule_musikrotation)
        client = self._get_client(self.user_admin)
        res = client.post(
            self._url("notes"),
            self._create_random_note_content(timeslot_id=timeslot.id),
            format="json",
        )
        self.assertEqual(res.status_code, 201)
